
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "lpc407x_8x_177x_8x.h"
#include "systime.h"


// abstract:
// Here we will send a periodic message. The interval between two messages is tracked
// by checking timer 0. Timer 0 is initialized by HW_Init().
// The periodic message includes the digital inputs. A CAN-ID is catched to set the
// digital output (low-side switch). The low-side switch is connected to Din0 so
// activating Dout0 can be seen at the periodic message.


// message 43Ch is sent every 50 millis
#define	MSG_43C_PERIOD_US		50000


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;


// variables for cyclic message transmission
static SYSTIME_t  LastCycleMsg;
static CANTxMsg_t  CycleMsg;





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// configure the cyclic message
	CycleMsg.bufftype = CAN_BUFFER_TX_MSG;
	CycleMsg.id = 0x43C;
	CycleMsg.dlc = CAN_LEN3_DLC;
	CycleMsg.msgtype = CAN_MSGTYPE_STANDARD;
	CycleMsg.data16[0] = 0;
	
	LastCycleMsg = SYSTIME_NOW;
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		SYSTIME_t  timenow;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 334h to control the low-side switch
			if ( RxMsg.id == 0x334  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				uint32_t  dig_out;
				
				
				// default OFF
				dig_out = 0;
				
				if ( RxMsg.data8[0] & 1)
				{
					// switch load to GND e.g. relay
					dig_out |= 1;
				}
			
				HW_SetDOUT ( &dig_out);
			}
			
			// forward message to CAN2
			CAN_Write ( CAN_BUS2, &RxMsg);
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
		
		
		// send a message every MSG_43C_PERIOD_US
		timenow = SYSTIME_NOW;
		
		if ( SYSTIME_DIFF ( LastCycleMsg, timenow) >= MSG_43C_PERIOD_US)
		{
			uint32_t  dig_in;
			
			
			// adjust last time of message by MSG_43C_PERIOD_US
			LastCycleMsg += MSG_43C_PERIOD_US;
			
			// read digital inputs
			HW_GetDIN ( &dig_in);
			
			// increment a simple counter
			CycleMsg.data16[0] += 1;
			
			// digital inputs at byte 2. Pull Din0..Din2 to GND
			CycleMsg.data8[2] = dig_in;
			
			// send
			CAN_Write ( CAN_BUS2, &CycleMsg);
		}
	}
}
