
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "serial.h"
#include "ser_user.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// abstract:
// Here we will forward CAN data to the serial port. Additionally we receive some data from
// the serial port to fill up a CAN message for transmission.


// size for user text buffer received from partner like terminal program or other device
#define	USER_TEXT_SIZE		30


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;

// variables for serial reception
static uint8_t txt_idx;
static char user_text[USER_TEXT_SIZE];





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// init serial
	SER_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// init variables
	txt_idx = 0;
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		CANTxMsg_t  TxMsg;
		SERResult_t  serstat;
		uint8_t  ser_read;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 00000650h to send on serial as ascii
			if ( RxMsg.id == 0x650  &&  RxMsg.msgtype == CAN_MSGTYPE_EXTENDED)
			{
				char  textbuff[30];
				uint32_t  textlen;
				
				
				// convert a 15-bit signal to ASCII text
				textlen = snprintf ( &textbuff[0], 30, "RPM = %d\n", RxMsg.data16[0] & 0x7FFF);
				
				// send on serial to terminal
				SER_Write ( SER_PORT1, &textbuff[0], textlen);
			}
			
			// catch ID 611h to send on serial as binary
			else if ( RxMsg.id == 0x611  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				SER_Write ( SER_PORT1, &RxMsg.data8[0], RxMsg.dlc);
			}
			
			else
			{
				// forward message to CAN2
				CAN_Write ( CAN_BUS2, &RxMsg);
			}
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
		
		
		// check input from serial. Modify code for your own needs. Check buffer for overflows if needed.
		serstat = SER_Read ( SER_PORT1, &user_text[txt_idx], USER_TEXT_SIZE - txt_idx, &ser_read);
		
		if ( serstat == SER_ERR_OK)
		{
			// data read from serial
			if ( user_text[txt_idx] == '\n')
			{
				// new line character, do nothing
			}
			
			else if ( user_text[txt_idx] == '\r')
			{
				// user string complete
				
				user_text[txt_idx] = 0;		// terminate with null
				
				if ( strncmp ( user_text, "fuel=", 5) == 0)
				{
					// new value for fuel level. Write a text like:
					// fuel=34
					// to a terminal and the router will send ID 450h  4  22h 00h 00h 00h
					TxMsg.bufftype = CAN_BUFFER_TX_MSG;
					TxMsg.id   = 0x450;
					TxMsg.msgtype = CAN_MSGTYPE_STANDARD;
					TxMsg.dlc  = CAN_LEN4_DLC;
					
					// convert ascii text to int value (basis 10)
					TxMsg.data32[0] = strtoul ( &user_text[5], NULL, 10);
					
					// send
					CAN_Write ( CAN_BUS1, &TxMsg);
				}
				
				txt_idx = 0;
			}
			
			else
			{
				// wait for more characters
				txt_idx += ser_read;
			}
		}
	}
}
