
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "hardware_user.h"
#include "eeprom.h"


// abstract:
// This example shows basic usage of the on-board SPI Flash. The user can use up to
// 32 MBit / 64 MBit of flash memory for e.g. calibration data. See PCAN-Router FD
// user manual for flash sizes. The function HW_UserInit() from
// hardware_user.c is called by HW_Init() implicit so you must not call this manually.
// For more info about the NXP LPC spifilib see www.lpcware.com


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;

// one page to save data from SPI Flash
uint32_t  SpifiBuff[64];





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 3B5h to modify data from SPI Flash
			if ( RxMsg.id == 0x3B5  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				// read one page of data
				spifiDevRead ( pSpifi, 0x1000, SpifiBuff, 256);
				
				// move some bytes to the CAN message
				RxMsg.data32[0] = SpifiBuff[0];
				RxMsg.data32[1] = SpifiBuff[1];
				
				// modify page buffer
				SpifiBuff[0] += 2;
				SpifiBuff[1] += 3;
				
				// erase 4K block #1 at address 0x1000
				spifiDevEraseSubBlock ( pSpifi, 1);
				
				// write page back to address 0x1000
				spifiDevPageProgram ( pSpifi, 0x1000, SpifiBuff, 256);
			}

			else if ( RxMsg.id == 0x400  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				uint32_t  i;


				for ( i = 0; i < 8192; i++)
				{
					spifiDevEraseSubBlock ( pSpifi, i);
				}
			}

			else if ( RxMsg.id == 0x500  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				uint32_t  i, t;


				t = 0xFFFFFFFF;

				for ( i = 0; i < 4032; i++)
				{
					EEPROM_Write(i, &t, 1);
				}

				EEPROM_FlushCache();

				RxMsg.id += 5;
			}

			// forward message to CAN2
			CAN_Write ( CAN_BUS2, &RxMsg);
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
	}
}
