
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "lpc407x_8x_177x_8x.h"
#include "systime.h"


// abstract:
// This example shows how to use the PCANFlash software during firmware runtime.
// The firmware will check some PCANFlash messages.


#define		VERSION_MAIN		1
#define		VERSION_SUB			0

#define		FW_DAY				0x25
#define		FW_MONTH				0x06
#define		FW_YEAR				0x15


// proto to prevent warning
static void main_check_flashmode ( CANRxMsg_t  *msg);


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;

// this is for PCANFlash handling
static uint8_t  send_detect_answer, module_id;
static SYSTIME_t  time_of_detect;





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}




// check some PCANFlash specific stuff
static void main_check_flashmode ( CANRxMsg_t  *msg)
{
	// evaluate some commands
	if ( msg->dlc == 3  &&  ( msg->data32[0] & 0x00FFFFFF) == 0x060080)
	{
		// detect
		send_detect_answer = 1;
		time_of_detect = SYSTIME_NOW;
	}
	
	else if ( msg->dlc == 7  &&  msg->data16[0] == 0xFF7F  &&  msg->data8[2] == module_id)
	{
		// command for me, check command
		if ( msg->data8[3] == 7  &&  msg->data8[4] == 0x55)
		{
			// invoke bootloader. See can_user.c for more info
			CAN_UserInvokeBootloader();
		}
		
		else if ( msg->data8[3] == 0)
		{
			// get status
			msg->dlc  = CAN_LEN6_DLC;
		
			msg->data8[0]	= 0x7F;
			msg->data8[1]	= 0xFF;
			msg->data8[2]	= module_id;
			msg->data8[3]	= 40;
			msg->data8[4]	= 40;
			msg->data8[5]	= 0;
			
			// send
			CAN_Write ( CAN_BUS1, msg);
		}
	}
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// get module ID from solder pins
	HW_GetModuleID ( &module_id);
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// inits
	send_detect_answer = 0;
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		CANTxMsg_t  TxMsg;
		SYSTIME_t  timenow;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 0x7E7 from PCANFlash Software
			if ( RxMsg.id == 0x7E7  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				// check flash mode from PCANFlash
				main_check_flashmode ( &RxMsg);
			}
			
			else
			{
				// forward message to CAN2
				CAN_Write ( CAN_BUS2, &RxMsg);
			}
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
		
		
		// get current time
		timenow = SYSTIME_NOW;
		
		// reply detect from PCANFlash software
		if ( send_detect_answer)
		{
			if ( SYSTIME_DIFF ( time_of_detect, timenow) >= ( 15000 * module_id))
			{
				// send answer for detect after module_id * 15 millis
				send_detect_answer = 0;
				
				TxMsg.bufftype = CAN_BUFFER_TX_MSG;
				TxMsg.id = 0x7E7;
				TxMsg.msgtype = CAN_MSGTYPE_STANDARD;
				TxMsg.dlc = CAN_LEN8_DLC;

				TxMsg.data8[0] = 0xE6;
				TxMsg.data8[1] = 0x40 | module_id;
				TxMsg.data8[2] = 0x06;
				TxMsg.data8[3] = FW_DAY;
				TxMsg.data8[4] = FW_MONTH;
				TxMsg.data8[5] = FW_YEAR;
				
				// version of firmware, bootloader will send it's own version !
				TxMsg.data8[6] = VERSION_MAIN << 5 | VERSION_SUB;
				TxMsg.data8[7] = 0;

				CAN_Write ( CAN_BUS1, &TxMsg);
			}
		}
	}
}
